@@ -1,5 +1,6 @@
 glmmML.fit <- function (X, Y,
                         weights = rep(1, NROW(Y)),
+                        cluster.weights = rep(1, NROW(Y)),
                         start.coef = NULL, 
                         start.sigma = NULL,
                         fix.sigma = FALSE,
@@ -149,6 +150,7 @@ glmmML.fit <- function (X, Y,
               as.double(start.coef),
               as.integer(cluster),
               as.double(weights),
+              as.double(cluster.weights),
               as.double(start.sigma),
               as.integer(fix.sigma),
               as.double(X), # Note CAREFULLY (03-01-09) AND 06-07-08 (back)!!!
